"""Poll - a sample Zope Product

Allows multiple choice voting.
"""

import Acquisition
import AccessControl
import OFS
from Globals import HTMLFile, MessageDialog, Persistent

class Poll(
	Acquisition.Implicit,
	Persistent,
	AccessControl.Role.RoleManager,
	OFS.SimpleItem.Item,
	):
	"""A Poll product. Allows for multiple choice voting.
	
	You can use a poll either by accessing it directly, or
	by embedding it in a Document with DTML.
	
	To embed a Poll use it's 'form' and 'results' methods.
	
	For example to display a poll's voting form::
		
		<!--#var expr="myPoll.form(myPoll,REQUEST,action=URL)"-->
		
	This will display 'myPoll's voting form. The 'action=URL' part
	tells the form to return this page after the vote is accepted.
	
	Here's how to display a poll's results::
	
		<!--#var expr="myPoll.result(myPoll,REQUEST)"-->
		
	This will display the stock poll results. You can also access
	a poll's internals directly by calling methods like 'votes_for'
	and 'percent_for' as you wish.
	
	To control voting access assign the roles you wish to the 'Vote'
	permission. There are currently no restrictions to prevent the
	same person from voting multiple times.
	"""
	
	icon='misc_/Poll/poll'
	
	meta_type='Poll'
	
	manage_options=(
		{'label':'Properties','action':'manage_main'},
		{'label':'View','action':''},
		{'label':'Security','action':'manage_access'},
		)

	__ac_permissions__=(
		('View management screens',('manage_tabs','manage_main')),
		('Change permissions',('manage_access',)),
		('Change Polls',('manage_edit','clear_votes'),('Manager',)),
		('Vote',('vote',),('Anonymous','Manager')),
		('View',('','form','results','total_votes','votes_for','percent_for')),
		)


	def __init__(self,id,title,question,choices):
		self.id=id
		self.title=title
		self.question=question
		self.choices=choices
		self.clear_votes()
	
	def clear_votes(self):
		"Delete all the votes"
		self.votes={}
		for choice in range(len(self.choices)):
			self.votes[choice]=0
	
	form=HTMLFile('pollForm',globals())

	results=HTMLFile('pollResults',globals())
	
	index_html=HTMLFile('poll',globals())
	
	manage_main=HTMLFile('pollEdit',globals())
	
	def vote(self,choice,action=None,REQUEST=None):
		"""Vote for a choice
		
		'choice' is an integer which represents a choice.
		'action' is an optional URL which the user will
		be taken to after they vote."""
		
		# this is done like this in order to allow Zope's
		# persistence system notice that we're updating 
		# a mutable attribute.
		votes=self.votes
		votes[choice]=votes[choice]+1
		self.votes=votes
		if action is None:
			action=''
		if REQUEST is not None:
			return MessageDialog(
				title='Vote Accepted',
				message='Your vote has been accepted.',
				action =action,
				)
		
	def total_votes(self):
		"Total number of votes cast"
		total=0
		for v in self.votes.values():
			total=total+v
		return total
		
	def votes_for(self,choice):
		"""Number of votes cast for a given choice
		
		'choice' is an integer which represents a choice.
		"""
		return self.votes[choice]

	def percent_for(self,choice):
		"""The percentage of votes cast for a given choice
		
		'choice' is an integer which represents a choice.
		"""
		return (self.votes[choice] / float(self.total_votes())) * 100

	def manage_edit(self,title,question,choices,reset_votes,REQUEST=None):
		"""Edit the poll's characteristics
		
		'title' is a string title.
		'question' is a string which represents the poll's question.
		'choices' is a list of at least 2 strings representing the poll's
		possible choices.
		'reset_votes' is a boolean. If true the votes will be reset.
		"""
		self.title=title
		self.question=question
		self.choices=[]
		for choice in choices:
			if choice:
				self.choices.append(choice)
		if len(self.choices) < 2:
			raise ValueError, "You must supply at least two valid choices."
		if reset_votes:
			self.clear_votes()
		if REQUEST is not None:
			return MessageDialog(
				title='Edited',
				message='<strong>%s</strong> has been edited.' % self.id,
				action ='./manage_main',
				)
		
		
addPollForm=HTMLFile('pollAdd',globals())

def addPoll(self,id,title,question,answers,REQUEST=None):
	"""Create a poll and install it in its parent Folder"
	
	The argument 'self' will be bound to the parent Folder.
	"""
	poll=Poll(id, title, question, answers)
	self._setObject(id, poll)
	if REQUEST is not None:
		return self.manage_main(self,REQUEST)
